// ==============================================================
//
// mpx_heatshield
// Control module for mpx_heatshield vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================


const double HEATSHIELD_MASS = 62;

static const DWORD ntdvtx_geardown = 4;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(   0, 2,0.5), 1e5, 1e4, 0.5, 0.5},
	{_V(-1.732,-1,0.5), 1e5, 1e4, 0.5, 0.5},
	{_V( 1.732,-1,0.5), 1e5, 1e4, 0.5, 0.5},
	{_V(   0,  0,  -0.5), 1e5, 1e4, 0.5}
};

//#define GREEN RGB(125, 255, 125)

//void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
//{
//	static const double step = RAD*22.5;
//	static const double istep = 1.0/step;
//	static const int nabsc = 17;
//	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};
//
//
//	aoa += PI;
//	int idx = max (0, min (15, (int)(aoa*istep)));
//	double d = aoa*istep - idx;
//	*cl = 0;//CL[idx] + (CL[idx+1]-CL[idx])*d;
//	*cm = 0.0;
//	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
//	
//	
//}

// 2. horizontal lift component (vertical stabiliser and body)

//void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
//{
//	static const double step = RAD*22.5;
//	static const double istep = 1.0/step;
//	static const int nabsc = 17;
//	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};
//
//	beta += PI;
//	int idx = max (0, min (15, (int)(beta*istep)));
//	double d = beta*istep - idx;
//	*cl = 0;//CL[idx] + (CL[idx+1]-CL[idx])*d;
//	*cm = 0.0;
//	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
//	
//}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class mpx_heatshield: public VESSEL3 {
public:
	mpx_heatshield (OBJHANDLE hVessel, int flightmodel);
	~mpx_heatshield ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	//void clbkPostCreation();
	//void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	//void clbkSaveState (FILEHANDLE scn);
	//void clbkLoadStateEx (FILEHANDLE scn, void *status);
	//int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	//bool clbkLoadGenericCockpit ();
	
private:

	//double timer;
	
};

mpx_heatshield::mpx_heatshield (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

mpx_heatshield::~mpx_heatshield ()
{

}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void mpx_heatshield::clbkSetClassCaps (FILEHANDLE cfg)
{
	
	//timer = 0; // self delete timer
	
	// physical vessel parameters
	SetEmptyMass (HEATSHIELD_MASS);
	SetSize (2);
	SetPMI(_V(2,2,2));
	SetCrossSections (_V(1,1,4));
	//SetTouchdownPoints ( _V(0,-5, -5), _V(-5, 3, -5), _V( 5, 3,-5));
	SetTouchdownPoints(tdvtx_geardown, ntdvtx_geardown);
	SetCW(1.5,1.5,1.5,1.5);
	SetRotDrag (_V(1,1,1));
	SetTrimScale(0.1);
	// camera parameters
	SetCameraOffset (_V(0,0,-2));
	
	SetMeshVisibilityMode(AddMesh("Phoenix_Mars_Lander/mpx_heatshield2"),MESHVIS_ALWAYS);


	

	//CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 1, 4, 0.16);
	//CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 1, 4, 0.16);

}

//void mpx_heatshield::clbkPostCreation ()
//{
//
//
//}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new mpx_heatshield (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (mpx_heatshield*)vessel;
}


//void mpx_heatshield::clbkSaveState (FILEHANDLE scn)
//{
//	char cbuf[256];
//
//	// default vessel parameters
//	VESSEL3::clbkSaveState(scn);
//
//	sprintf (cbuf, "%0.4f", timer);
//	oapiWriteScenario_string (scn, "ANMS", cbuf);
//
//
//
//}

//void mpx_heatshield::clbkLoadStateEx (FILEHANDLE scn, void *vs)
//{
//    char *line;
//	while (oapiReadScenario_nextline (scn, line)) {
//		if (!strnicmp (line, "ANMS", 4)) {
//		sscanf (line+4, "%lf", &timer);
//		} else {
//            ParseScenarioLineEx (line, vs);
//			// unrecognised option - pass to Orbiter's generic parser
//        }
//   }
//
//	
//
//}



//void mpx_heatshield::clbkPreStep(double SimT, double SimDT, double MJD)
//{
//	//timer = timer + SimDT;
//	//if(timer > 300)oapiDeleteVessel(GetHandle());
//
//}

