// ==============================================================
//
// mpx_solp
// Control module for mpx_solp vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================


const double SOLP_MASS = 82;

static const DWORD ntdvtx_geardown = 4;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(   0, 2,0.5), 1e5, 1e4, 0.5, 0.5},
	{_V(-1.732,-1,0.5), 1e5, 1e4, 0.5, 0.5},
	{_V( 1.732,-1,0.5), 1e5, 1e4, 0.5, 0.5},
	{_V(   0,  0,  -0.5), 1e5, 1e4, 0.5}
};

//#define GREEN RGB(125, 255, 125)

//void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
//{
//	static const double step = RAD*22.5;
//	static const double istep = 1.0/step;
//	static const int nabsc = 17;
//	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};
//
//
//	aoa += PI;
//	int idx = max (0, min (15, (int)(aoa*istep)));
//	double d = aoa*istep - idx;
//	*cl = 0;//CL[idx] + (CL[idx+1]-CL[idx])*d;
//	*cm = 0.0;
//	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
//	
//	
//}

// 2. horizontal lift component (vertical stabiliser and body)

//void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
//{
//	static const double step = RAD*22.5;
//	static const double istep = 1.0/step;
//	static const int nabsc = 17;
//	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};
//
//	beta += PI;
//	int idx = max (0, min (15, (int)(beta*istep)));
//	double d = beta*istep - idx;
//	*cl = 0;//CL[idx] + (CL[idx+1]-CL[idx])*d;
//	*cm = 0.0;
//	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
//	
//}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class mpx_solp: public VESSEL3 {
public:
	mpx_solp (OBJHANDLE hVessel, int flightmodel);
	~mpx_solp ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	//void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	//void clbkSaveState (FILEHANDLE scn);
	//void clbkLoadStateEx (FILEHANDLE scn, void *status);
	//int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	//bool clbkLoadGenericCockpit ();
	
private:

	//double timer;
	
};

mpx_solp::mpx_solp (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

mpx_solp::~mpx_solp ()
{

}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void mpx_solp::clbkSetClassCaps (FILEHANDLE cfg)
{
	
	//timer = 0; // self delete timer
	
	// physical vessel parameters
	SetEmptyMass (SOLP_MASS);
	SetSize (3);
	SetPMI(_V(2,2,2));
	SetCrossSections (_V(4,4,12));
	//SetTouchdownPoints ( _V(0,-5, -5), _V(-5, 3, -5), _V( 5, 3,-5));
	SetTouchdownPoints(tdvtx_geardown, ntdvtx_geardown);
	SetCW(1,1,1,1);
	SetRotDrag (_V(1,1,1));
	SetTrimScale(0.1);
	// camera parameters
	SetCameraOffset (_V(0,0,-2));
	
	SetMeshVisibilityMode(AddMesh("Phoenix_Mars_Lander/mpx_solp"),MESHVIS_ALWAYS);

	// reentry pstream
	PARTICLESTREAMSPEC rps = {
		0, 2.5, 20, 0, 0.001, 0.5, 4, 0.2, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1, PARTICLESTREAMSPEC::ATM_PLOG, 1e8, 5e15
	};
	AddReentryStream (&rps);
	

	//CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 1, 4, 0.16);
	//CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 1, 4, 0.16);

}

//void mpx_solp::clbkPostCreation ()
//{
//
//
//}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new mpx_solp (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (mpx_solp*)vessel;
}


//void mpx_solp::clbkSaveState (FILEHANDLE scn)
//{
//	char cbuf[256];
//
//	// default vessel parameters
//	VESSEL3::clbkSaveState(scn);
//
//	sprintf (cbuf, "%0.4f", timer);
//	oapiWriteScenario_string (scn, "ANMS", cbuf);
//
//
//
//}

//void mpx_solp::clbkLoadStateEx (FILEHANDLE scn, void *vs)
//{
//    char *line;
//	while (oapiReadScenario_nextline (scn, line)) {
//		if (!strnicmp (line, "ANMS", 4)) {
//		sscanf (line+4, "%lf", &timer);
//		} else {
//            ParseScenarioLineEx (line, vs);
//			// unrecognised option - pass to Orbiter's generic parser
//        }
//   }
//
//	
//
//}



void mpx_solp::clbkPreStep(double SimT, double SimDT, double MJD)
{
	if(GetAltitude(ALTMODE_MEANRAD) < 40000)oapiDeleteVessel(GetHandle());
	else if(GetAltitude(ALTMODE_MEANRAD) > (3*AU))oapiDeleteVessel(GetHandle());
}

